@@ -37,11 +37,11 @@ class AzureClient():
                 self.service.get_blob_to_path(self.bucket_name, blob.name, dst_folder + '/' + splitted_name[len(splitted_name) - 1])
 
     def _get_content_type(self, dst_file):
-        if dst_file.endswidth('/Manifest') or dst_file.endswidth('.ismc') or dst_file == 'Manifest':
+        if dst_file.endswith('/Manifest') or dst_file.endswith('.ismc') or dst_file == 'Manifest':
             content_settings = ContentSettings(content_type='text/xml')
-        elif dst_file.endswidth('.m3u8'):
+        elif dst_file.endswith('.m3u8'):
             content_settings = ContentSettings(content_type='application/x-mpegURL')
-        elif dst_file.endswidth('.mpd'):
+        elif dst_file.endswith('.mpd'):
             content_settings = ContentSettings(content_type='application/dash+xml')
         else:
             content_settings = None
