@@ -53,11 +53,11 @@ class GCloudStorageClient():
                 blob.download_to_filename(dst_folder + '/' + splitted_name[len(splitted_name) - 1])
 
     def _get_content_type(self, dst_file):
-        if dst_file.endswidth('/Manifest') or dst_file.endswidth('.ismc') or dst_file == 'Manifest':
+        if dst_file.endswith('/Manifest') or dst_file.endswith('.ismc') or dst_file == 'Manifest':
             content_settings = 'text/xml'
-        elif dst_file.endswidth('.m3u8'):
+        elif dst_file.endswith('.m3u8'):
             content_settings = 'application/x-mpegURL'
-        elif dst_file.endswidth('.mpd'):
+        elif dst_file.endswith('.mpd'):
             content_settings = 'application/dash+xml'
         else:
             content_settings = None
